module bellek_yonetimi_2;

import core.memory;

/* GC.realloc gibi işler. Ondan farklı olarak, belleğin
 * uzatıldığı durumda eklenen baytları sıfırlar. */
void * boşOlarakUzat(
        void * yer,
        size_t eskiUzunluk,
        size_t yeniUzunluk,
        GC.BlkAttr bellekNitelikleri = GC.BlkAttr.NONE,
        const TypeInfo türBilgisi = null) {
    /* Asıl işi GC.realloc'a yaptırıyoruz. */
    yer = GC.realloc(yer, yeniUzunluk,
                     bellekNitelikleri, türBilgisi);

    /* Eğer varsa, yeni eklenen bölümü sıfırlıyoruz. */
    if (yeniUzunluk > eskiUzunluk) {
        import core.stdc.string;

        auto eklenenYer = yer + eskiUzunluk;
        const eklenenUzunluk = yeniUzunluk - eskiUzunluk;

        memset(eklenenYer, 0, eklenenUzunluk);
    }

    return yer;
}
struct Dizi(T) {
    T * yer;          // Elemanların bulunduğu yer
    size_t sığa;      // Toplam kaç elemanlık yer olduğu
    size_t uzunluk;   // Eklenmiş olan eleman adedi

    /* Belirtilen numaralı elemanı döndürür */
    T eleman(size_t numara) {
        import std.string;
        enforce(numara < uzunluk,
                format("%s numara yasal değil", numara));

        return *(yer + numara);
    }

    /* Elemanı dizinin sonuna ekler */
    void ekle(T eleman) {
        writefln("%s numaralı eleman ekleniyor", uzunluk);

        if (uzunluk == sığa) {
            /* Yeni eleman için yer yok; sığayı arttırmak
             * gerekiyor. */
            size_t yeniSığa = sığa + (sığa / 2) + 1;
            sığaArttır(yeniSığa);
        }

        /* Elemanı en sona yerleştiriyoruz */
        *(yer + uzunluk) = eleman;
        ++uzunluk;
    }

    void sığaArttır(size_t yeniSığa) {
        writefln("Sığa artıyor: %s -> %s",
                 sığa, yeniSığa);

        const eskiUzunluk = sığa * T.sizeof;
        const yeniUzunluk = yeniSığa * T.sizeof;

        /* Bu bölgeye yerleştirilen bayt değerlerinin
         * tesadüfen başka değişkenlerin göstergeleri
         * sanılmalarını önlemek için NO_SCAN belirtecini
         * kullanıyoruz. */
        yer = cast(T*)boşOlarakUzat(
            yer, eskiUzunluk, yeniUzunluk, GC.BlkAttr.NO_SCAN);

        sığa = yeniSığa;
    }
}
import std.stdio;
import core.memory;
import std.exception;

// ...

void main() {
    auto dizi = Dizi!double();

    size_t adet = 10;

    foreach (i; 0 .. adet) {
        double elemanDeğeri = i * 1.1;
        dizi.ekle(elemanDeğeri);
    }

    writeln("Bütün elemanlar:");

    foreach (i; 0 .. adet) {
        write(dizi.eleman(i), ' ');
    }

    writeln();
}
