module es_zamanli_5;

import std.stdio;
import std.random;
import std.string;
import std.concurrency;
import core.thread;

struct Yer {
    int satır;
    int sütun;

    string toString() {
        return format("%s,%s", satır, sütun);
    }
}

struct Hareket {
    Yer nereden;
    Yer nereye;

    string toString() {
        return ((nereden == nereye)
                ? format("%s (durgun)", nereden)
                : format("%s -> %s", nereden, nereye));
    }
}

class Robot {
    string görünüm;
    Duration dinlenmeSüresi;

    this(string görünüm, Duration dinlenmeSüresi) {
        this.görünüm = görünüm;
        this.dinlenmeSüresi = dinlenmeSüresi;
    }

    override string toString() {
        return format("%s(%s)", görünüm, dinlenmeSüresi);
    }
}

/* 0,0 noktası etrafında rasgele bir yer döndürür */
Yer rasgeleYer() {
    return Yer(uniform!"[]"(-10, 10), uniform!"[]"(-10, 10));
}

/* Verilen değerin en fazla bir adım ötesinde bir değer
 * döndürür */
int rasgeleAdım(int şimdiki) {
    return şimdiki + uniform!"[]"(-1, 1);
}

/* Verilen Yer'in komşusu olan bir Yer döndürür; çapraz
 * komşusu olabileceği gibi tesadüfen aynı yer de olabilir. */
Yer rasgeleKomşu(Yer yer) {
    return Yer(rasgeleAdım(yer.satır),
               rasgeleAdım(yer.sütun));
}

struct İş {
    size_t robotNumarası;
    Yer başlangıç;
    Duration dinlenmeSüresi;
}

struct HareketMesajı {
    size_t robotNumarası;
    Hareket hareket;
}

void gezdirici(İş iş) {
    Yer nereden = iş.başlangıç;

    while (true) {
        Thread.sleep(iş.dinlenmeSüresi);

        Yer nereye = rasgeleKomşu(nereden);
        Hareket hareket = Hareket(nereden, nereye);
        nereden = nereye;

        ownerTid.send(HareketMesajı(iş.robotNumarası, hareket));
    }
}

void main() {
    /* Farklı hızlardaki robotlar */
    Robot[] robotlar = [ new Robot("A",  600.msecs),
                         new Robot("B", 2000.msecs),
                         new Robot("C", 5000.msecs) ];

    /* Her birisi için bir iş parçacığı başlatılıyor */
    foreach (robotNumarası, robot; robotlar) {
        spawn(&gezdirici, İş(robotNumarası,
                             rasgeleYer(),
                             robot.dinlenmeSüresi));
    }

    /* Artık hareket bilgilerini işçilerden toplamaya
     * başlayabiliriz */
    while (true) {
        auto mesaj = receiveOnly!HareketMesajı();

        /* Bu robotla ilgili yeni bilgiyi çıkışa
         * yazdırıyoruz */
        writefln("%s %s",
                 robotlar[mesaj.robotNumarası],
                 mesaj.hareket);
    }
}
