module fiberler_6;

import std.stdio;
import std.string;
import std.format;
import std.exception;
import std.conv;
import std.array;
import core.thread;

struct Kullanıcı {
    string isim;
    string eposta;
    uint yaş;
}

/* Bu alt sınıf kullanıcı kayıt akışını temsil eder. */
class KayıtAkışı : Fiber {
    /* Bu akış için en son okunmuş olan veri. */
    string veri_;

    /* Kullanıcı nesnesi kurmak için gereken bilgi. */
    string isim;
    string eposta;
    uint yaş;

    this() {
        /* Fiberin başlangıç noktası olarak 'başlangıç' üye
         * işlevini belirtiyoruz. */
        super(&başlangıç);
    }

    void başlangıç() {
        /* İlk girilen veri isimdir. */
        isim = veri_;
        Fiber.yield();

        /* İkinci girilen veri e-postadır. */
        eposta = veri_;
        Fiber.yield();

        /* Sonuncu veri yaştır. */
        yaş = veri_.to!uint;

        /* Bu noktada Kullanıcı nesnesi oluşturacak bütün
         * veriyi toplamış bulunuyoruz. 'Fiber.yield()' ile
         * duraksamak yerine artık işlevin sonlanmasını
         * istiyoruz. (Burada açıkça 'return' deyimi de
         * olabilirdi.) Bunun sonucunda bu fiberin durumu
         * Fiber.State.TERM değerini alır. */
    }

    /* Bu nitelik işlevi çağıranın veri girmesi içindir. */
    void veri(string yeniVeri) {
        veri_ = yeniVeri;
    }

    /* Bu nitelik işlevi kurulan nesneyi çağırana vermek
     * içindir. */
    Kullanıcı kullanıcı() const {
        return Kullanıcı(isim, eposta, yaş);
    }
}

/* Belirli bir akış için girişten okunmuş olan veriyi temsil
 * eder. */
struct AkışVerisi {
    size_t numara;
    string yeniVeri;
}

/* Belirtilen satırdan akış verisi okur. */
AkışVerisi akışVerisiOku(string satır) {
    size_t numara;
    string yeniVeri;

    const adet =
        satır.formattedRead!" %s %s"(numara, yeniVeri);

    enforce(adet == 2,
            format("Geçersiz veri: '%s'.", satır));

    return AkışVerisi(numara, yeniVeri);
}

void main() {
    Kullanıcı[] kullanıcılar;
    KayıtAkışı[] akışlar;

    bool bitti_mi = false;

    while (!bitti_mi) {
        write("> ");
        string satır = readln.strip;

        switch (satır) {
        case "merhaba":
            /* Yeni bağlanan kullanıcı için yeni akış
             * oluşturalım. */
            akışlar ~= new KayıtAkışı();

            writefln("%s numaralı akış başladı.",
                     akışlar.length - 1);
            break;

        case "son":
            /* Programdan çıkalım. */
            bitti_mi = true;
            break;

        default:
            /* Girilen satırı akış verisi olarak kullanmaya
             * çalışalım. */
            try {
                auto kullanıcı = veriİşle(satır, akışlar);

                if (!kullanıcı.isim.empty) {
                    kullanıcılar ~= kullanıcı;
                    writefln("'%s' eklendi.", kullanıcı.isim);
                }

            } catch (Exception hata) {
                writefln("Hata: %s", hata.msg);
            }
            break;
        }
    }

    writeln("Güle güle.");
    writefln("Kullanıcılar:\n%(  %s\n%)", kullanıcılar);
}

/* Girilen verinin ait olduğu fiberi belirler, yeni verisini
 * bildirir, ve o fiberin işleyişini kaldığı yerden devam
 * ettirir. Eğer girilen son veri üzerine akış sonlanmışsa,
 * üyeleri geçerli değerlerden oluşan bir Kullanıcı nesnesi
 * döndürür. */
Kullanıcı veriİşle(string satır, KayıtAkışı[] akışlar) {
    const akışVerisi = akışVerisiOku(satır);
    const numara = akışVerisi.numara;

    enforce(numara < akışlar.length,
            format("Geçersiz numara: %s.", numara));

    auto akış = akışlar[numara];

    enforce(akış.state == Fiber.State.HOLD,
            format("Akış %s işletilebilir durumda değil.",
                   numara));

    /* Akışa yeni verisini bildir. */
    akış.veri = akışVerisi.yeniVeri;

    /* Akışı kaldığı yerden devam ettir. */
    akış.call();

    Kullanıcı kullanıcı;

    if (akış.state == Fiber.State.TERM) {
        writefln("Akış %s tamamlandı.", numara);

        /* Dönüş değerine yeni oluşturulan kullanıcıyı ata. */
        kullanıcı = akış.kullanıcı;

        /* Sonrası için fikir: 'akışlar' dizisinin artık işi
         * bitmiş olan bu elemanı yeni bağlanacak olan
         * kullanıcılar için kullanılabilir. Ancak, önce
         * 'akış.reset()' ile tekrar başlatılabilir duruma
         * getirilmesi gerekir. */
    }

    return kullanıcı;
}
