module sablonlar_ayrintili_12;

import std.stdio;
import std.format;
import std.string;

/* İki boyutlu bir int dizisi gibi işler. */
struct Matris {
private:

    int[][] satırlar;

    /* İndekslerle belirlenen satır ve sütun aralığı bilgisini
     * bir araya getirir. */
    struct Aralık {
        size_t baş;
        size_t son;
    }

    /* Satır ve sütun aralıklarıyla belirlenen alt matrisi
     * döndürür. */
    Matris altMatris(Aralık satırAralığı, Aralık sütunAralığı) {
        writeln(__FUNCTION__);

        int[][] dilimler;

        foreach (satır; satırlar[satırAralığı.baş ..
                                 satırAralığı.son]) {

            dilimler ~= satır[sütunAralığı.baş ..
                              sütunAralığı.son];
        }

        return Matris(dilimler);
    }

public:

    this(size_t yükseklik, size_t genişlik) {
        writeln(__FUNCTION__);

        satırlar = new int[][](yükseklik, genişlik);
    }

    this(int[][] satırlar) {
        writeln(__FUNCTION__);

        this.satırlar = satırlar;
    }

    void toString(void delegate(const(char)[]) hedef) const {
        hedef.formattedWrite!"%(%(%5s %)\n%)"(satırlar);
    }

    /* Belirtilen değeri matrisin bütün elemanlarına atar. */
    Matris opAssign(int değer) {
        writeln(__FUNCTION__);

        foreach (satır; satırlar) {
            satır[] = değer;
        }

        return this;
    }

    /* Belirtilen işleci ve değeri her elemana uygular ve
     * sonucu o elemana atar. */
    Matris opOpAssign(string işleç)(int değer) {
        writeln(__FUNCTION__);

        foreach (satır; satırlar) {
            mixin ("satır[] " ~ işleç ~ "= değer;");
        }

        return this;
    }

    /* Belirtilen boyutun uzunluğunu döndürür. */
    size_t opDollar(size_t boyut)() const
            if (boyut <= 1) {
        writeln(__FUNCTION__);

        static if (boyut == 0) {
            /* Sıfırıncı boyutun uzunluğu isteniyor;
             * 'satırlar' dizisinin uzunluğudur. */
            return satırlar.length;

        } else {
            /* Birinci boyutun uzunluğu isteniyor; 'satırlar'
             * dizisinin elemanlarının uzunluğudur. */
            return satırlar.length ? satırlar[0].length : 0;
        }
    }

    /* 'baş' ve 'son' ile belirlenen aralığı ifade eden bir
     * nesne döndürür.
     *
     * Not: Bu gerçekleştirmede 'boyut' parametresi
     * kullanılmıyor olsa da, bu bilgi başka bir tür için
     * yararlı olabilir. */
    Aralık opSlice(size_t boyut)(size_t baş, size_t son)
            if (boyut <= 1) {
        writeln(__FUNCTION__);

        return Aralık(baş, son);
    }

    /* Parametrelerle belirlenen alt matrisi döndürür. */
    Matris opIndex(A...)(A parametreler)
            if (A.length <= 2) {
        writeln(__FUNCTION__);

        /* Bütün elemanları temsil eden aralıklarla
         * başlıyoruz. Böylece opIndex'in parametresiz
         * kullanımında bütün elemanlar kapsanırlar. */
        Aralık[2] aralıklar = [ Aralık(0, opDollar!0),
                                Aralık(0, opDollar!1) ];

        foreach (boyut, p; parametreler) {
            static if (is (typeof(p) == Aralık)) {
                /* Bu boyut için 'matris[baş..son]' gibi
                 * aralık belirtilmiş; parametreyi olduğu gibi
                 * aralık olarak kullanabiliriz. */
                aralıklar[boyut] = p;

            } else static if (is (typeof(p) : size_t)) {
                /* Bu boyut için 'matris[i]' gibi tek konum
                 * değeri belirtilmiş; kullanmadan önce tek
                 * uzunluklu aralık oluşturmak gerekiyor. */
                aralıklar[boyut] = Aralık(p, p + 1);

            } else {
                /* Bu işlevin başka bir türle çağrılmasını
                 * beklemiyoruz. */
                static assert(
                    false, format("Geçersiz indeks türü: %s",
                                  typeof(p).stringof));
            }
        }

        /* 'parametreler'in karşılık geldiği alt matrisi
         * döndürüyoruz. */
        return altMatris(aralıklar[0], aralıklar[1]);
    }

    /* Belirtilen değeri belirtilen elemanlara atar. */
    Matris opIndexAssign(A...)(int değer, A parametreler)
            if (A.length <= 2) {
        writeln(__FUNCTION__);

        Matris altMatris = opIndex(parametreler);
        return altMatris = değer;
    }

    /* Belirtilen işleci ve değeri belirtilen elemanlara
     * uygular ve sonuçları yine aynı elemanlara atar. */
    Matris opIndexOpAssign(string işleç, A...)(int değer,
                                               A parametreler)
            if (A.length <= 2) {
        writeln(__FUNCTION__);

        Matris altMatris = opIndex(parametreler);
        mixin ("return altMatris " ~ işleç ~ "= değer;");
    }
}

/* Dizgi halinde belirtilen ifadeyi işletir ve hem işlemin
 * sonucunu hem de matrisin yeni durumunu yazdırır. */
void işlet(string ifade)(Matris m) {
    writefln("\n--- %s ---", ifade);
    mixin ("auto sonuç = " ~ ifade ~ ";");
    writefln("sonuç:\n%s", sonuç);
    writefln("m:\n%s", m);
}

void main() {
    enum yükseklik = 10;
    enum genişlik = 8;

    auto m = Matris(yükseklik, genişlik);

    int sayaç = 0;
    foreach (satır; 0 .. yükseklik) {
        foreach (sütun; 0 .. genişlik) {
            writefln("%s / %s ilkleniyor",
                     sayaç + 1, yükseklik * genişlik);

            m[satır, sütun] = sayaç;
            ++sayaç;
        }
    }

    writeln(m);

    işlet!("m[1, 1] = 42")(m);
    işlet!("m[0, 1 .. $] = 43")(m);
    işlet!("m[0 .. $, 3] = 44")(m);
    işlet!("m[$-4 .. $-1, $-4 .. $-1] = 7")(m);

    işlet!("m[1, 1] *= 2")(m);
    işlet!("m[0, 1 .. $] *= 4")(m);
    işlet!("m[0 .. $, 0] *= 10")(m);
    işlet!("m[$-4 .. $-2, $-4 .. $-2] -= 666")(m);

    işlet!("m[1, 1]")(m);
    işlet!("m[2, 0 .. $]")(m);
    işlet!("m[0 .. $, 2]")(m);
    işlet!("m[0 .. $ / 2, 0 .. $ / 2]")(m);

    işlet!("++m[1..3, 1..3]")(m);
    işlet!("--m[2..5, 2..5]")(m);

    işlet!("m[]")(m);
    işlet!("m[] = 20")(m);
    işlet!("m[] /= 4")(m);
    işlet!("(m[] += 5) /= 10")(m);
}
